using namespace std;

#include <iostream>
#include <unordered_set>
#include <unordered_map>
#include "testlib.h"

typedef long long ll;
typedef pair<int, int> edge;

const int e5 = 100000;
const int max_n = 3 * e5;
const int max_m = 3 * e5;
const int max_m_g1 = 20;
const int max_deg_g2 = 2;

bool is_tree(const vector<vector<int>> &g, vector<bool> &us, int v, int p) {
    if (us[v]) {
        return false;
    }
    us[v] = true;
    for (int u : g[v]) {
        if (u != p && !is_tree(g, us, u, v)) {
            return false;
        }
    }
    return true;
}

int main(int argc, char *argv[]) {
    registerValidation(argc, argv);

    int n = inf.readInt(2, max_n, "n");
    inf.readSpace();
    int m = inf.readInt(1, max_m, "m");
    inf.readEoln();
    if (validator.group() == "1") {
        ensuref(m <= max_m_g1, "g1 m = %d > %d", m, max_m_g1);
    }

    vector <vector<int>> graph(n + 1);
    set <edge> e_set;
    vector<int> degs(n + 1);
    for (int i = 0; i < m; ++i) {
        int u = inf.readInt(1, n, "u_i");
        inf.readSpace();
        int v = inf.readInt(1, n, "v_i");
        ensuref(u != v, "u_%d == v_%d", u, v);
        ensuref(e_set.find(make_pair(u, v)) == e_set.end(), "edge %d-%d already was", u, v);
        e_set.emplace(u, v);
        e_set.emplace(v, u);
        graph[v].push_back(u);
        graph[u].push_back(v);
        degs[u]++;
        degs[v]++;
        inf.readEoln();
    }
    inf.readEof();

    if (validator.group() == "2") {
        for (int i = 1; i <= n; ++i) {
            ensuref(0 <= degs[i] && degs[i] <= max_deg_g2, "g2: deg_%d = %d", i, degs[i]);
        }
    }
    if (validator.group() == "3") {
        for (int i = 1; i <= n; ++i) {
            ensuref(degs[i] == n - 1, "g3: is not completed graph, deg_%d = %d", i, degs[i]);
        }
    }
    if (validator.group() == "4") {
        vector<bool> used(n + 1);
        for (int i = 1; i <= n; ++i) {
            if (!used[i]) {
                ensuref(is_tree(graph, used, i, 0), "g4: is not tree, component with node %d", i);
            }
        }
    }
    return 0;
}
